package com.fjhx.file.service;

import com.fjhx.file.entity.FileInfo;
import com.fjhx.file.entity.FileInfoSelectDto;
import com.fjhx.file.entity.FileInfoVo;
import com.fjhx.file.entity.ObsFile;
import com.fjhx.file.entity.SingDto;
import com.fjhx.file.entity.SingVo;
import com.ruoyi.common.core.service.BaseService;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 文件表 服务类
 * </p>
 *
 * @author
 * @since 2023-03-14
 */
public interface FileInfoService extends BaseService<FileInfo> {

    /**
     * 获取前端直传签名
     */
    SingVo getSing(SingDto dto);

    /**
     * 文件表列表
     */
    Map<Long, List<FileInfoVo>> getList(FileInfoSelectDto dto);

    /**
     * 保存文件
     *
     * @param obsFileList  文件列表
     * @param businessId   业务id
     * @param businessType 业务文件类型
     */
    void saveFile(List<ObsFile> obsFileList, Long businessId, Integer businessType);

    /**
     * 保存文件
     *
     * @param obsFileList  文件列表
     * @param businessId   业务id
     * @param businessType 业务文件类型
     */
    void saveAllFile(List<ObsFile> obsFileList, Long businessId, Integer businessType);

    /**
     * 更新文件
     *
     * @param obsFileList  文件列表
     * @param businessId   业务id
     * @param businessType 业务文件类型
     */
    void editFile(List<ObsFile> obsFileList, Long businessId, Integer businessType);

    /**
     * 删除文件
     *
     * @param businessId   业务id
     * @param businessType 业务文件类型
     */
    void removeFile(Long businessId, Integer businessType);

    /**
     * 删除文件
     *
     * @param businessId 业务id
     */
    void removeFile(Long businessId);

    /**
     * 删除文件
     *
     * @param businessIdList 业务id列表
     */
    void removeFile(List<Long> businessIdList);

    /**
     * 判断文件是否已经绑定业务 是将文件复制一份并保存
     *
     * @param fileList     文件列表
     * @param businessId   业务id
     * @param businessType 业务文件类型
     */
    void copyFileAndSave(List<ObsFile> fileList, Long businessId, Integer businessType);

    /**
     * 交换业务ID
     *
     * @param newBusinessId 新业务id
     * @param oldBusinessId 旧业务id
     */
    void exchangeBusinessId(Long oldBusinessId, Long newBusinessId);

}
